io8255_a    equ 230h       ; 定义8255端口A地址为230h
io8255_b    equ 231h       ; 定义8255端口B地址为231h
io8255_c    equ 232h       ; 定义8255端口C地址为232h
io8255_k    equ 233h       ; 定义8255控制端口地址为233h
io8253_0    equ 210h       ; 定义8253计数器0地址为210h
io8253_1    equ 211h       ; 定义8253计数器1地址为211h
io8253_k    equ 213h       ; 定义8253控制端口地址为213h
io8259_0    equ 200h       ; 定义8259中断控制器0地址为200h
io8259_1    equ 201h       ; 定义8259中断控制器1地址为201h
AD_START    equ   220H      ; 定义AD转换启动信号地址为220h
AD_OE       equ   221H      ; 定义AD输出使能信号地址为221h
AD_EOI      equ   222H      ; 定义AD结束标志地址为222h

; 数据段定义
data        segment
    count1  dw  0               ; 中断计数器，记录已经过的时间
    portcl  db  0ch,14h,21h,22h,0ffh  ; 红绿灯的状态码数组（5个状态字）
    input   db  ?               ; 存储从AD转换得到的电位器数据
    buf     db  0               ; 存储电机的相序
data        ends

; 栈段定义
st_ssg      segment stack
    db  256 dup(?)            ; 为栈分配256字节
st_ssg      ends

; 代码段定义
code        segment
    assume  cs:code, ds:data, ss:st_ssg
start:
    cli                       ; 关中断，防止在初始化过程中出现中断
    mov ax, data              ; 将数据段地址赋值给ax寄存器
    mov ds, ax                ; 将ax寄存器的值赋给ds寄存器，设置数据段

    ; 初始化8259中断控制器
    mov dx, io8259_0           ; 将io8259_0的地址赋值给dx寄存器
    mov al, 00010011B          ; 控制字1，配置8259控制寄存器
    out dx, al
    mov dx, io8259_1           ; 将io8259_1的地址赋值给dx寄存器
    mov al, 00001000B          ; 控制字2，配置8259控制寄存器
    out dx, al
    mov al, 00000001B          ; 控制字4，配置8259控制寄存器
    out dx, al

    ; 设置中断向量表
    push    ds
    mov ax, 0
    mov ds, ax
    lea ax, cs:int_proc        ; 将中断服务程序地址赋给ax寄存器
    mov si, 08h                ; MIR0类型码08h（IRQ0-7）
    add si, si                 ; 计算地址偏移
    add si, si
    mov ds:[si], ax            ; 将中断服务程序的地址存入中断向量表
    push    cs
    pop ax
    mov ds:[si+2], ax          ; 存储cs地址
    pop ds
    in  al, dx
    and al, 11111110b
    out dx, al

    ; 初始化8253定时器
    mov ax, 0
    mov al, 00110110b          ; 设置8253控制字，配置定时器工作模式
    mov dx, io8253_k
    out dx, al
    mov ax, 10000              ; 设置定时器初始计数值
    mov dx, io8253_0
    out dx, al
    mov al, ah
    out dx, al

    ; 初始化8255并配置端口
    mov dx, io8255_k
    mov al, 10001000b          ; 配置8255的控制字
    out dx, al

    mov buf, 33h               ; 设置电机的初始相序
    sti                        ; 开中断，允许中断发生

    ; 启动AD转换过程
lop1:   
    mov dx, AD_START
    out dx, al                 ; 启动AD转换
l1: 
    mov dx, AD_EOI
    in  al, dx
    test    al, 01h            ; 检查AD转换是否完成
    jz  l1                      ; 如果没有完成，等待
    mov dx, AD_OE
    in  al, dx
    mov input, al              ; 读取AD转换结果

out1:   
    mov al, buf
    mov dx, io8255_c
    out dx, al                 ; 将电机相序数据输出到8255端口C

lop:    
    call    delay              ; 调用延时子程序
    mov dx, io8255_c
    in  al, dx                 ; 读取端口C的数据

sam:    
    test    al, 80h            ; 检查数据的特定位
    jnz nx0
    mov al, buf
    ror al, 1                  ; 顺时针旋转，电机正转
    jmp nx1
nx0: 
    mov al, buf
    rol al, 1                  ; 逆时针旋转，电机反转
nx1:    
    mov dx, io8255_c
    out dx, al
    mov buf, al

light:  
    call    digital            ; 调用数字显示控制子程序

    mov dx, io8255_b
greeen1: 
    cmp count1, 500            ; 判断是否已到达5秒，主干道通行
    ja  yellow1
    mov al, portcl
    out dx, al
    jmp ok

yellow1: 
    cmp count1, 700            ; 判断是否已到达2秒，主干道黄灯
    ja  greeen2
    mov al, portcl[1]
    out dx, al
    jmp ok

greeen2: 
    cmp count1, 1000           ; 判断是否已到达3秒，从干道通行
    ja  yellow2
    mov al, portcl[2]
    out dx, al
    jmp ok

yellow2: 
    cmp count1, 1200           ; 判断是否已到达2秒，从干道黄灯
    ja  finish
    mov al, portcl[3]
    out dx, al
    jmp ok

finish: 
    mov count1, 0              ; 一次循环结束后，重置计数器
ok: 
    call    digital
    jmp lop1

; 数字显示控制子程序
digital proc
begin:
    mov dx, io8255_a
zhu5:   
    cmp count1, 100
    ja  zhu4
    mov al, 6Dh                
    out dx, al
    jmp exit

zhu4:   
    cmp count1, 200
    ja  zhu3
    mov al, 66h               
    out dx, al
    jmp exit

zhu3:   
    cmp count1, 300
    ja  zhu2
    mov al, 4Fh                ; 显示数字'4'
    out dx, al
    jmp exit

zhu2:   
    cmp count1, 400
    ja  zhu1
    mov al, 5Bh                ; 显示数字'5'
    out dx, al
    jmp exit

zhu1:   
    cmp count1, 500
    ja  zhu22
    mov al, 06h                ; 显示数字'0'
    out dx, al
    jmp exit

zhu22:  
    cmp count1, 600
    ja  zhu11
    mov al, 5Bh                ; 显示数字'5'
    out dx, al
    jmp exit

zhu11:  
    cmp count1, 700
    ja  zhi3
    mov al, 06h                ; 显示数字'0'
    out dx, al
    jmp exit

zhi3:   
    cmp count1, 800
    ja  zhi2
    mov al, 4Fh                ; 显示数字'4'
    out dx, al
    jmp exit

zhi2:   
    cmp count1, 900
    ja  zhi1
    mov al, 5Bh                ; 显示数字'5'
    out dx, al
    jmp exit

zhi1:   
    cmp count1, 1000
    ja  zhi22
    mov al, 06h                ; 显示数字'0'
    out dx, al
    jmp exit

zhi22:  
    cmp count1, 1100
    ja  zhi11
    mov al, 5Bh                ; 显示数字'5'
    out dx, al
    jmp exit

zhi11:  
    cmp count1, 1200
    ja  exit
    mov al, 06h                ; 显示数字'0'
    out dx, al
exit:   
    ret
digital endp

; 延时子程序
delay   proc    near
    push    ax
    push    cx
    push    dx
    mov dl, input   
x1: 
    mov cx, 0ffh
x2: 
    loop    x2                 ; 循环延时
    dec dl
    jnz x1
    pop dx
    pop cx
    pop ax
    ret
delay   endp

; 中断服务程序
int_proc    proc    far
    push    ax
    push    dx
int1:   
    inc count1                 ; 中断计数器加1
    mov al, 20h
    mov dx, io8259_0
    out dx, al                 ; 发送EOI信号给8259中断控制器
    pop dx
    pop ax
    sti                        ; 开中断
    iret                       ; 中断返回
int_proc    endp

code    ends
end start